import { ref, watch, onBeforeMount, Ref, computed } from "vue";
import * as API from "@/api";
import { Howl, Howler } from "howler";

//答题模式切换
export function useTopicMode() {
  const answerTypeList = ref([
    { name: "顺序练习" },
    { name: "随机练习" },
    { name: "背题模式" },
  ]);
  const currentType = ref(0);
  const typeParams = ref({
    order: true, //顺序练习
    answerShow: false, //背题模式
  });
  watch(currentType, (currentVal) => {
    switch (currentVal) {
      case 0:
        typeParams.value.order = true;
        typeParams.value.answerShow = false;
        break;
      case 1:
        typeParams.value.order = false;
        typeParams.value.answerShow = false;
        break;
      case 2:
        typeParams.value.order = true;
        typeParams.value.answerShow = true;
        break;
    }
  });
  return {
    answerTypeList,
    currentType,
    typeParams,
  };
}

//语音设置
export function useAudioSet(currentAnswerIndex: Ref<number>) {
  const aotuPlayFlag = ref(false);

  let sound: Howl;
  /**
   * 播放音频
   * @param audioUrl
   */
  const audioPlay = (audioUrl: string | string[]) => {
    audioPause();
    sound = new Howl({
      src: audioUrl,
    });
    sound.once("load", function () {
      sound.play();
    });
    if (typeof audioUrl === "object") {
      sound.once("end", () => {
        sound = new Howl({
          src: audioUrl[1],
        });
        sound.once("load", function () {
          sound.play();
        });
      });
    }
  };

  /**
   * 自动读题
   */
  const issueAudioPlay = () => {};

  /**
   * 停止播放
   */
  const audioPause = () => {
    sound && sound.pause();
  };

  //音频模块end
  const aotuPlaySet = () => {
    aotuPlayFlag.value = !aotuPlayFlag.value;
    if (aotuPlayFlag.value) issueAudioPlay();
    if (!aotuPlayFlag.value) audioPause();
  };

  onBeforeMount(() => {
    watch(currentAnswerIndex, () => {
      if (aotuPlayFlag.value) issueAudioPlay(); //自动读题
    });
  });

  return {
    aotuPlayFlag,
    issueAudioPlay,
    audioPause,
    aotuPlaySet,
    audioPlay,
  };
}

//题目展示逻辑
export function useTopicShow(
  trueNum: Ref<number>,
  falseNum: Ref<number>,
  idIndex: Ref<number>,
  total: Ref<number>,
  isJump: Ref<boolean>,
  query: any
) {
  const topicList = ref([]);

  //API请求接口数据
  onBeforeMount(async () => {
    const topicListApiRes = await API.getTopicList({
      ...query,
      pageNum: 1,
      pageSize: 10,
    });
    topicList.value = topicListApiRes.list;
    idIndex.value = currentAnswerIndex.value;
    total.value = topicListApiRes.total;
  });

  //当前题目数据
  const currentAnswerIndex = ref(0);
  onBeforeMount(() => {
    watch(currentAnswerIndex, async () => {
      if (currentAnswerIndex.value === topicList.value.length - 1) {
        const topicListApiRes = await API.getTopicList({
          ...query,
          pageNum: 1,
          pageSize: 10,
        });
        topicList.value = topicList.value.concat(topicListApiRes.list);
      }
      idIndex.value = currentAnswerIndex.value;
    });
  });
  const currentAnswer: any = computed(() => {
    return topicList.value[currentAnswerIndex.value];
  });

  //上一题,下一题
  const currentAnswerIndexBack = () => {
    if (topicList.value.length == 0) return;
    currentAnswerIndex.value =
      (currentAnswerIndex.value - 1 + topicList.value.length) %
      topicList.value.length;
  };
  const currentAnswerIndexGo = () => {
    if (topicList.value.length == 0) return;
    currentAnswerIndex.value =
      (currentAnswerIndex.value + 1) % topicList.value.length;
  };

  //选择答案后
  const userAnswerChange = () => {
    currentAnswer.value.optsBack = currentAnswer.value.opts.map(
      (val: String) => {
        let status;
        if (currentAnswer.value.answer.includes(val)) {
          status = 1;
        } else {
          status = 0;
        }
        if (currentAnswer.value.userAnswer.includes(val)) {
          status += 2;
        }
        return { opt: val, status };
      }
    );
    if (
      JSON.stringify(currentAnswer.value.answer) ==
      JSON.stringify(currentAnswer.value.userAnswer)
    ) {
      console.log("答案正确");
      currentAnswer.value.isTrue = true;
      trueNum.value++;
      if (isJump.value)
        setTimeout(() => {
          currentAnswerIndexGo();
        }, 200);
    } else {
      console.log("错误");
      currentAnswer.value.isTrue = false;
      falseNum.value++;
    }
  };
  const topicType = (type: Number) => {
    switch (type) {
      case 0:
        return "判断题";
      case 1:
        return "单选题";
      case 2:
        return "多选题";
    }
  };

  return {
    topicList,
    currentAnswerIndex,
    currentAnswer,
    currentAnswerIndexBack,
    currentAnswerIndexGo,
    userAnswerChange,
    topicType,
  };
}
